#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "util.h"

const char *program;

void *xmalloc(size_t len)
{
  void *p = malloc(len);

  if (!p) {
    perror(program);
    exit(1);
  }

  return p;
}

struct disk *allocate_disk(const struct diskfmt *fmt)
{
  struct disk *disk = xmalloc(sizeof(struct disk));
  size_t size = fmt->sectors << 8;
  int i, starter;
  int secperclust = 1 << fmt->cluster_shift;

  memset(disk, 0, sizeof *disk);

  disk->fmt = fmt;
  disk->image = xmalloc(size);
  disk->bitmap = disk->image + (fmt->syssec << 8);
  disk->mfd.disk = disk;
  disk->mfd.bitmap = disk->bitmap;
  disk->mfd.dirsec = disk->bitmap + 2*256;
  disk->mfd.block = 0;		/* This is the MFD */

  /* Post-formatting content? */
  memset(disk->image, 0x40, size);

  /* Create empty bitmap */
  memset(disk->bitmap, 0, 239);
  for (i = fmt->sectors >> fmt->cluster_shift; i < 239*8; i++)
    mark_used(disk, i);		/* Mark nonexistent clusters busy */
  memset(disk->bitmap+239, 0x01, fmt->dirsecs);	/* MFD bitmap */
  disk->bitmap[0xff] = 0x00;

  /* Mark the first 16/32 sectors used */
  starter = (fmt->syssec + 2 + fmt->dirsecs + secperclust - 1)
    >> fmt->cluster_shift;
  for (i = 0; i < starter; i++)
    mark_used(disk, i);

  /* Copy bitmap to backup empty bitmap (= bad block map) */
  memcpy(disk->bitmap + 256, disk->bitmap, 256);

  /* Create MFD directory sectors */
  for (i = 0; i < fmt->dirsecs; i++) {
    unsigned char *p = disk->mfd.dirsec + ((i + fmt->syssec) << 8);
    memset(p, 0x00, 16);
    memset(p+16, 0xFF, 240);
  }

  return disk;
}
